library graph;

import 'dart:async';

import 'package:graphql/client.dart';
import './graph_queries.dart' as queries;

const String BASE_URL = 'https://graph.fuse.io/subgraphs/name/fuseio';
const String SUB_GRAPH = 'fuse-qa';

class Graph {
  GraphQLClient _clientFuse;
  GraphQLClient _clientRopsten;
  GraphQLClient _clientMainnet;

  Graph({String url, String subGraph}) {
    _clientFuse = GraphQLClient(
        link: HttpLink(uri: '${url ?? BASE_URL}/${subGraph ?? SUB_GRAPH}'),
        cache: InMemoryCache());
    _clientRopsten = GraphQLClient(
        link: HttpLink(uri: '${url ?? BASE_URL}/fuse-ropsten'),
        cache: InMemoryCache());
    _clientMainnet = GraphQLClient(
        link: HttpLink(uri: '${url ?? BASE_URL}/fuse-mainnet'),
        cache: InMemoryCache());
  }

  Future<dynamic> getCommunityByAddress(String communityAddress) async {
    final QueryOptions options = QueryOptions(
      documentNode: gql(queries.getCommunityByAddress),
      variables: <String, dynamic>{
        'address': communityAddress,
      },
    );
    QueryResult result = await _clientFuse.query(options);
    if (result.hasException) {
      throw 'Error! Get community request failed - communityAddress: $communityAddress';
    } else {
      return result.data["communities"][0];
    }
  }

  Future<dynamic> getTokenOfCommunity(String communityAddress) async {
    final QueryOptions options = QueryOptions(
      documentNode: gql(queries.getTokenOfCommunity),
      variables: <String, dynamic>{
        'address': communityAddress,
      },
    );
    QueryResult result = await _clientFuse.query(options);
    if (result.hasException) {
      throw 'Error! Get token of community request failed - communityAddress: $communityAddress';
    } else {
      return result.data["tokens"][0];
    }
  }

  Future<bool> isCommunityMember(
      String accountAddress, String entitiesListAddress) async {
    _clientFuse.cache.reset();
    final QueryOptions options = QueryOptions(
      documentNode: gql(queries.isCommunityMember),
      variables: <String, dynamic>{'address': accountAddress, 'entitiesList': entitiesListAddress},
    );
    QueryResult result = await _clientFuse.query(options);
    if (result.hasException) {
      throw 'Error! Is community member request failed - accountAddress: $accountAddress, entitiesListAddress: $entitiesListAddress';
    } else {
      return result.data["communityEntities"].length > 0;
    }
  }

  Future<BigInt> getTokenBalance(
      String accountAddress, String tokenAddress) async {
    _clientFuse.cache.reset();
    final QueryOptions options = QueryOptions(
      documentNode: gql(queries.getTokenBalance),
      variables: <String, dynamic>{
        'account': accountAddress,
        'token': tokenAddress
      },
    );
    QueryResult result = await _clientFuse.query(options);
    if (result.hasException) {
      throw 'Error! Get token balance request failed - accountAddress: $accountAddress, tokenAddress: $tokenAddress';
    } else {
      try {
        return BigInt.from(
            num.parse(result.data["accounts"][0]["tokens"][0]["balance"]));
      } catch (RangeError) {
        return BigInt.from(0);
      }
    }
  }

  Future<dynamic> getReceivedTransfers(
      String accountAddress, String tokenAddress, {int fromBlockNumber, int toBlockNumber}) async {
    _clientFuse.cache.reset();
    final QueryOptions options = QueryOptions(
      documentNode: gql(queries.getReceivedTransfers),
      variables: <String, dynamic>{
        'account': accountAddress,
        'token': tokenAddress,
        'n': 20,
      },
    );

    if (fromBlockNumber != null) {
      options.variables['fromBlockNumber'] = fromBlockNumber;
    }
    if (toBlockNumber != null) {
      options.variables['toBlockNumber'] = toBlockNumber;
    }
    QueryResult result = await _clientFuse.query(options);
    if (result.hasException) {
      throw 'Error! Get transfers request failed - accountAddress: $accountAddress, tokenAddress: $tokenAddress';
    } else {
      List transfers = [];

      for (dynamic t in result.data['transfersIn']) {
        transfers.add({
          "blockNumber": num.parse(t["blockNumber"]),
          "data": t["data"] ?? null,
          "from": t["from"],
          "id": t["id"],
          "to": t["to"],
          "tokenAddress": t["tokenAddress"],
          "txHash": t["txHash"],
          "value": t["value"],
          "type": "RECEIVE",
          "status": "CONFIRMED"
        });
      }

      transfers.sort((a, b) => b["blockNumber"].compareTo(a["blockNumber"]));
      return {"count": transfers.length, "data": transfers};
    }
  }
  Future<dynamic> getTransfers(
      String accountAddress, String tokenAddress, {int fromBlockNumber, int toBlockNumber}) async {
    _clientFuse.cache.reset();

    final QueryOptions options = QueryOptions(
      documentNode: gql(queries.getReceivedTransfers),
      variables: <String, dynamic>{
        'account': accountAddress,
        'token': tokenAddress,
        'n': 20,
      },
    );
    if (fromBlockNumber != null) {
      options.variables['fromBlockNumber'] = fromBlockNumber;
    }
    if (toBlockNumber != null) {
      options.variables['toBlockNumber'] = toBlockNumber;
    }
    QueryResult result = await _clientFuse.query(options);
    if (result.hasException) {
      throw 'Error! Get transfers request failed - accountAddress: $accountAddress, tokenAddress: $tokenAddress';
    } else {
      List transfers = [];

      for (num i = 0; i < result.data["transfersIn"].length; i++) {
        dynamic t = result.data["transfersIn"][i];
        transfers.add({
          "blockNumber": num.parse(t["blockNumber"]),
          "data": t["data"] ?? null,
          "from": t["from"],
          "id": t["id"],
          "to": t["to"],
          "tokenAddress": t["tokenAddress"],
          "txHash": t["txHash"],
          "value": t["value"],
          "type": "RECEIVE",
          "status": "CONFIRMED"
        });
      }

      for (num i = 0; i < result.data["transfersOut"].length; i++) {
        dynamic t = result.data["transfersOut"][i];
        transfers.add({
          "blockNumber": num.parse(t["blockNumber"]),
          "data": t["data"] ?? null,
          "from": t["from"],
          "id": t["id"],
          "to": t["to"],
          "tokenAddress": t["tokenAddress"],
          "txHash": t["txHash"],
          "value": t["value"],
          "type": "SEND",
          "status": "CONFIRMED"
        });
      }
      transfers.sort((a, b) => b["blockNumber"].compareTo(a["blockNumber"]));
      return {"count": transfers.length, "data": transfers};
    }
  }
}
